! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: mosback_mod
!
! Authors:  Brendan Perry - NASA/GSFC, brendan.h.perry@nasa.gov (f95 rewrite)
!           Kip Kuntz - NASA/GSFC/JHU (original algorithm)
!           Steven L. Snowden - NASA/GSFC (original algorithm)
!
! Description: Extended Source Analysis Software EMOS background calculator
!
! Comments:
!
! $Id: mosback_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!  Creates particle background spectra and images for EMOS
!
!  mosback was part of the XMMGOF-ESAS package and it produces model
!  particle background spectra and images (in detector coordinates)
!  from the output files of the perl script mosspectra (old: mos-spectra)
!
!  Input: OLD:
!    prefix: Exposure ID prefix - e.g., prefix=1S001
!    caldb: QPB data directory - e.g., caldb=/XMM/esas/caldb 
!    elow: Lower energy limit for the energy band in eV, if only spectra
!        are desired set to 0 - e.g., elow=0 or elow=400
!    ehigh: Upper energy limit for the energy band in eV, if only spectra
!        are desired sereading t to 0 - e.g., ehigh=0 or ehigh=1250
!    ccds boolean array (list) can use T/F/0/1 (which ccd?)
!    withimg: boolean: write image file or only spectra(set to F)?
!    withqdp: boolean: write qdp files?
!    chisqfl: boolean: r*4 spec with unc?
!    ***clobber: deprecated, uses SAS_CLOBBER
!    ***diag   : deprecated, uses SAS_VERBOSITY and withqdp
!
! OLD CALL:
! mos_back prefix=1S001 caldb=/Users/bhperry/XMM/CCF/esas/ diag=0 elow=300 ehigh=1100 \
! ccd1=1 ccd2=1 ccd3=1 ccd4=1 ccd5=1 ccd6=1 ccd7=1
!
!   NEW:
!     instead of prefix, use inspecfile, which can have any name
!     plus rmffile, which can also have any name
!     extracts keywords and forms other filenames on the fly that are unique
!     qpbfile instead of caldb, so file can be anywhere with any name, but
!     this may be replaced by CAL call (2020)
!     Do NOT mistake QPB for QDP. QDP is the text output file for QDP/pgplot,
!     which should be replaced by python plotters after SAS-20 (BHP)
!
! oldExample: mosback prefix=1S003 caldb=/XMM/esas/caldb diag=0 elow=0
!                 ehigh=0 ccd1=1 ccd2=1 ccd3=1 ccd4=1 ccd5=1 ccd6=1 ccd7=1
! newExamples:
!    mosback inspecfile=mos1spec.fit outspecfile=mos1specout withimg=T imgfile=mos1-im.fits \
!     elow=0 ehigh=0 ccds="1 1 1 1 1 1 1" chisqfl=T
!
module mosback_mod

  use dal
  use param 
  use errorHandling
  use esas_mod      ! in goflib, formerly in subsn (SS)
  use gofutils_mod  ! in goflib, formerly in marq_mod
  use gofmatrix_mod ! in goflib, formerly in marq_mod
  use gofio_mod     ! in goflib, formerly in subsn (SS)

  implicit none

  private

  ! task parameter names
  ! filenames except for inspecfile are all optional. they will be derived
  ! from 'prefix', created out of the inst + expidstr keywords in inspecfile
  ! if the user does not enter their own filenames
  
  character(len=*), parameter :: &
       inspecpar    = "inspecfile", &  ! was derived from prefix, i.e. 1S001/2U006
       outspecpar   = "outspecfile", & ! was derived, will be if 'default'
       rmfsetpar    = "rmffile", &     ! was: parsed from prefix
       withqdppar   = "withqdp",&      ! write out QDP diagnostic file?
       qdppar       = "qdpfile",&      ! output QDP file name
       withimgpar   = "withimg",&      ! write out FITS image
       imgpar       = "imgfile",&      ! output FITS IMAGE 
       elowpar      = "elow", &        ! low energy cutoff
       ehighpar     = "ehigh", &       ! high energy cutoff
       ccdspar      = "ccds", &        ! ccds to apply
       chisqflpar   = "chisqfl"        ! Use chi squared statistics,write out rate/error?
  !
  ! bhp: NEW: no diag, replaced with SAS_VERBOSITY, etc
  !
  public mosback

contains

  !---------------------------------------------------------------------
  subroutine mosback
    !---------------------------------------------------------------------
    !
    ! SAS datatypes:
    !
    type(DataSetT) :: inSPEC_S, inRMF_S, bkimdet_S, specccdo_S, &
         imgobs_S, imgfwc_S, speccoro_S, speccorc_S, specfwco_S, &
         specfwcc_S, obimdet_S, backspec_S, QPB_S, outSPEC_S
    type(ArrayT)   :: obsimg_A, imgfwc_A, obimdet_A, inSPEC_A, out_A
    type(TableT)   :: inSPEC_T, speccoro_T, speccorc_T, &
         specfwco_T, specfwcc_T, specccdo_T, backspec_T, out_T
    type(ColumnT) :: channel_C, spectrum_C, staterr_C, countsR32_C, &
         countsI32_C
    !
    character (len=256) :: inSPEC_F, outSPEC_F, inRMF_F, outQDP_F, &
         specfwcc_F, speccoro_F, specfwco_F, specccdo_F, &
         obimdet_F, bkimdet_F, imgobs_F, imgfwc_F, qdpratio_F,&
         qdpaug_F, qdpback_F, qdpbacki_F, qdpbacka_F, bridge_F, &
         bkimget_F, backspecout_F, obsspec_F, outIMG_F
    character(len=1) :: str_ccd
    character(len=2) :: instE, instO, instS, instfwco, instfwcc, instcoro, &
         instimg, instcorr, instccdo
    character(len=5) :: celow, cehigh, expidimg, expidcoro, expidcorc, &
         expidfwcc, expidfwco, expidccdo, expidS, expidO, expidE, inst
    character(len=6) :: filterO, filterS, filtercoro, filterccdo, &
         filterfwcc, filterfwco
    character(len=8) :: prefix,ctype(2), cunit(2), &
         ctypeimg(2), cunitimg(2)
    character(len=10) :: obsidfwco, obsidfwcc, &
         obsidcoro, obsidcorc, obsidccdo, obsidO, obsidS
    character(len=24) :: title
    character(len=25) :: smodeO, smodeS, smodefwco, smodefwcc, &
         smodecoro, smodecorc, smodeimg, smodeccdo
    character(len=80) :: tstring
    character(len=256) :: QPB_F, tempdir

    integer :: lun, revlO, rmfchans, typeA
    integer :: chansO, revolutO, revlcoro, revlfwco, revlfwcc, &
         revlccdo, chansccdo, chanscoro, chansfwcc, chansfwco, &
         frmtimeccdo, frmtimecoro, frmtimefwcc, frmtimefwco, &
         frmtimeO, frmtimeS, typeF
    integer :: out(2), sim_chips(7), plc_al(2), plc_alsi(2),&
         plc_aua(2), plc_aub(2), plc_auc(2), plc_hi(2), plc_lo(2)
    integer :: ehigh, elow, ehighO, elowO, i, i1, i2, ichan, ii, &
         it, ites, itest, j, jj, mosnum, n, nnn, n_real, n_sims, nord, &
         lunbridge, lunqdpback, status
    integer :: ehighccdo, ehighfwcc, ehighfwco, &
         ehighcoro, elowcoro, elowccdo, elowfwcc, elowfwco, elowS, ehighS
    integer, dimension(2) :: band = (/0, 0/)
    integer :: ipix = 1
    integer :: nels = 13
    integer :: smo_fact = 4
    integer, parameter :: specchans = 2400
    integer, parameter :: xGood = 780, yGood = 780 !adjust here
    integer :: xSize, ySize, i_ok
    integer, dimension(:), allocatable :: chansout, ispecout
    integer, dimension(:,:), allocatable :: imgfwc, imgout, obimdet, imgobs
    integer, dimension(13) :: mine = (/118, 379, 460, 766, 1108, &
         1213, 1318, 1423, 1513, 1633, 1753, 1993, 2359/)
    integer, dimension(13) :: maxe = (/237, 417, 765, 1050, 1155, &
         1245, 1380, 1470, 1575, 1680, 1875, 2205, 2397/)
    integer :: lunoqdp, lunqdp, lunaug, lunbacki, lunbacka, lunratio
    integer(kind=int16), dimension(:), pointer :: chansO_IP, chansfwco_IP, &
         chanscoro_IP, chansfwcc_IP, speccoro_IP
    integer(kind=int16), dimension(:,:), pointer :: aDI16
    integer(kind=int32), dimension(:), pointer :: speccoroU_JP, speccoro_JP, &
         specfwcc_JP, specfwccU_JP, specO_JP, specS_JP, dummy_JP, specfwco_JP
    integer(kind=int32), dimension(:,:), pointer :: &
         obimdet_JP, imgobs_JP, imgfwc_JP
    real(kind=single), dimension(:,:), pointer :: aDR32
    real(kind=double), dimension(:,:), pointer :: aDR64
    !
    logical :: withqdp, withimg, chisqfl, warndie, did, mos1
    logical :: ccds(7) ! bhp: one array for all ccds needed
    real(single), allocatable, dimension(:,:) :: matrix
    real(kind=single), dimension(:), pointer :: speccoroU_RP, speccoro_RP, &
         specfwcc_RP, specfwccU_RP, specfwcoU_RP, specO_RP, &
         specOU_RP, specSU_RP, specS_RP, specfwco_RP, dummy_RP
    real(kind=single), parameter :: amsca = 1440000.0
    real(kind=double), allocatable, dimension (:) :: tempe, temps, tempu
    real(kind=single), dimension(3) :: aug_scal = (/0.01, 1.0, 0.0/)
    real(kind=double), dimension(10) :: coef ! whyyyyy 10???
    real(kind=single), allocatable, dimension (:) :: aug_spec, &
         emax, emin, ener, fc1, modl, fillin, mk1, &
         oc1,  temp_bot, temp_top, uc1, &
         specobs, specobs_U, speccoro, speccoro_U, &
         areaobs_tot, backobs_tot, expoobs_tot, &
         areacoro_tot, backcoro_tot, expocoro_tot, &
         areacorc_tot, backcorc_tot, expocorc_tot, &
         areafwco_tot, areafwcc_tot, backfwco_tot, backfwcc_tot,&
         expofwco_tot, expofwcc_tot, specfwcc, specfwcc_U, &
         specfwco, specfwco_U, maskout, specout, specout_U
    real(kind=single), allocatable, dimension(:,:) :: image, mask_tot, outim, &
         specobs_tot, specobs_tot_U, speccoro_tot, speccoro_tot_U, ratio_tot, &
         specfwcc_tot, specfwcc_tot_U, specfwco_tot, specfwco_tot_U, &
         maskcoro_tot, maskcorc_tot
    real(kind=single) :: ymax = 0, ymin = 1.e8 
    real(kind=single) aug_expo, aug_rate, avtot, areaO, areaS, backO, backS, &
         bandl, bandh, hard, high, back, expo, lim_area, low, ntot, rate, &
         t, t1, t2, tmodcounts, tot, tot1, tot2, areacoro, backcoro, expocoro, &
         areacorc, backcorc, expocorc, backobs, areaobs, expoobs, backccdo, &
         areaccdo, expoccdo, backfwco, backfwcc, areafwco, areafwcc, expofwcc, &
         expofwco, rmocfob, rmocfori, rmocftb, expoO, expoS
    real(kind=single) :: aug_time = 1.0e6 ! why hardcoded???!!!
    ! 
    inSPEC_F = stringParameter(inspecpar) ! OLD: mos1S001-obj.pi NEW:mos1S001-fovt.pi
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(inSPEC_F)),inSPEC_S,1,status,warndie) ! 1 = readonly
    !
    ! Get critical header keywords from input spectrum
    !
    ! BHP: Only have to read file once and keep values in memory.
    !      The old version opened and read it with fitsio 3 times.
    !      Read mos1S002-fovt.pi (contains SPECTRUM extension,
    !      channel/counts columns, 2400 elements (typically))
    !
    ! SS: Read the observation chip spectra to get the exposure
    !
    write(errstr,*)"Reading input spectrum: ",trim(adjustl(inSPEC_F))
    call message(AppMsg,Always) ! change to Verbose???!!!
    status = 0
    warndie = .TRUE. ! if not found, unreadable, die
    typeF = 2 ! 1 = EVLI, 2 = SPEC, 3 = IMG, many of these are dummies
    call read_header_info(typeF,inSPEC_S,inSPEC_T,inSPEC_A,instO,obsidO,&
         expidO,smodeO,frmtimeO,filterO,revlO,backO,areaO,expoO,elowO,&
         ehighO,chansO,warndie)
    !
    ! set prefix for default filename creation and other usage
    !
    inst = 'EMOS1'
    if (instO .eq. 'M1') then 
       mos1 = .TRUE.
       prefix = "mos1" // trim(adjustl(expidO)) ! mos1S001
       QPB_F = 'emos1_qpb.fits.gz'
    elseif (instO .eq. 'M2') then
       mos1 = .FALSE.
       inst = "EMOS2"
       prefix = "mos2" // trim(adjustl(expidO)) ! mos2S002
       QPB_F = 'emos2_qpb.fits.gz'
    else
       call error ("notM1orM2","Instrument in input SPEC must be M1 or M2")
    endif
    ! Check for QPB file. If doesn't exist or can't be read, no need to keep going
    write(errstr,*)"Running saslocate to find correct QPB cal file"
    call message(AppMsg,Always) ! change to Noisy!!!??? 
    call execute_command_line('saslocate lib/data/esas_data > temp.txt')
    lun = GetLun()
    open (unit=lun,file="temp.txt",status='old',access='sequential',&
         form='formatted',action='read',iostat=i_ok)
    if (i_ok == 0) then
       read(lun,'(A)') tempdir
       close (lun)
       call unlink ("temp.txt")
    else
       call error("saslocateBad","saslocate for QPB file failed!")
    end if
    QPB_F = trim(adjustl(tempdir)) // "/" // trim(adjustl(QPB_F))
    write(errstr,*)"QPB calibration file:"
    call message(AppMsg,Always) ! change to Noisy!!!??? 
    write(errstr,*)trim(adjustl(QPB_F))
    call message(AppMsg,Always) ! change to Noisy!!!???
    !
    ! can this be defaulted to 0 or something then retrieved automatically
    ! from ELOW/EHIGH keywords take from the input file (which had those
    ! written to it upstream, i.e. in mosspectra???!!!)
    elow      = intParameter(elowpar) 
    ehigh     = intParameter(ehighpar)
    write(celow,'(I5)') elow
    write(cehigh,'(I5)') ehigh
    outSPEC_F   = stringParameter(outspecpar) ! !!!??? put something here
    if (trim(adjustl(outSPEC_F)) .eq. 'default') then ! celow/cehigh???!!!
       outSPEC_F = trim(adjustl(prefix)) // "-back.pi" ! for now ???!!!
    endif
    ! optional parameter; if none give on command line, create one based on prefix
    ! from input spectral file
    inRMF_F = stringParameter(rmfsetpar) ! mos1S001.rmf
    if (trim(adjustl(inRMF_F)) .eq. 'default') then
       inRMF_F = trim(adjustl(prefix)) // ".rmf"
    end if
    !
    do i = 1, 7 
       ccds(i) = booleanParameter(ccdspar,i-1)
    end do
    withqdp = booleanParameter(withqdppar) ! write diag qdp files?
    if (withqdp) then
       outQDP_F = stringParameter(qdppar) ! OLD: mos1S001.qdp NEW: mos1S001-backspec.qdp
       if (trim(adjustl(outQDP_F)) .eq. 'default') then
          outQDP_F = trim(adjustl(prefix)) // "-backspec.qdp"
       end if
    end if
    withimg = booleanParameter(withimgpar) ! write out image files?
    if (withimg) then ! from mosspectra???!!! will overwrite!!!
       outIMG_F = stringParameter(imgpar) 
       if (trim(adjustl(outIMG_F)) .eq. 'default') then
          outIMG_F = trim(adjustl(prefix)) // "-backim.fits" ! check this ???!!!
       end if
    end if
    chisqfl = booleanParameter(chisqflpar) ! int vs real + unc output?
    !
    call printparams (inSPEC_F,outSPEC_F,inRMF_F,withqdp,outQDP_F,withimg,&
         outIMG_F,elow,ehigh,ccds,revlO,obsidO)
    !
    lim_area = 0.0 ! bhp: set here, but never changes???!!!
    !
    write(errstr,*) "Integer or real+unc output spectrum (chisqfl)?",chisqfl
    call message(AppMsg,Always)

    write(errstr,*) &
         "  WARNING: If chip 3, 4 or 6 are missing, spurious "
    call message(AppMsg,Always)
    write(errstr,*) &
         "  gold features can arise in the 2.04-2.33, 9.38-9.98, "
    call message(AppMsg,Always)
    write(errstr,*) "  and 11.03-11.82 keV regions."
    call message(AppMsg,Always)
    !
    !  SS: Check to see whether MOS1 CCD#6 is missing
    !  (this is done by the DAL/SOC now) check this???!!!
    ! extract keywords from obs spec file header/extension
    !
    !  SS: Read the observation chip spectra to get the exposure
    !  bhp: done above now
    !
    ! mosnum, inst, obsid, expid, submode, revl, and chansO should be
    ! consistent for all opened files files below. Tests should decide
    ! to abort if, e.g., corner spectrum is obsid 2348760201 and obs
    ! spectrum is obsid 1234567890, right???
    ! is this still necessary ???!!!
    !
    !  SS: Do some setup
    !  This sets the outer CCDs which will be used as cognates
    !  for the central CCD
    !
    n = 0
    if(mos1) then
       if(ccds(2)) then
          n = n + 1
          sim_chips(n) = 2
       endif
       if(ccds(3)) then
          n = n + 1
          sim_chips(n) = 3
       endif
       if(ccds(4)) then
          if(revlO < 2383) then
             n = n + 1
             sim_chips(n) = 4
          endif
       endif
       if(ccds(5)) then
          n = n + 1
          sim_chips(n) = 5
       endif
       if(ccds(6)) then
          n = n + 1
          sim_chips(n) = 6
       endif
       if(ccds(7)) then
          n = n + 1
          sim_chips(n) = 7
       endif
       n_sims = n
       n = n + 1
       do while (n .le. 7)
          sim_chips(n) = 0
          n = n + 1
       enddo
    else ! mos2
       if(ccds(2)) then
          n = n + 1
          sim_chips(n) = 2
       endif
       if(ccds(3)) then
          n = n + 1
          sim_chips(n) = 3
       endif
       if(ccds(4)) then
          n = n + 1
          sim_chips(n) = 4
       endif
       if(ccds(5)) then
          n = n + 1
          sim_chips(n) = 5
       endif
       if(ccds(6)) then
          n = n + 1
          sim_chips(n) = 6
       endif
       if(ccds(7)) then
          n = n + 1
          sim_chips(n) = 7
       endif
       n_sims = n
       n = n + 1
       do while (n .le. 7)
          sim_chips(n) = 0
          n = n + 1
       enddo
    end if
    do i=1,7
       write(str_ccd,'(I1)')i
       write(errstr,*)' sim_chips(' // str_ccd // ') = ',sim_chips(i)
       call message(AppMsg,Always) ! change to Noisy???!!!
    end do
    !
    bandl = elow / 1000.0    ! whyyyyy???!!! 
    bandh = ehigh / 1000.0   ! 
    !
    !  Get information about the object spectrum
    !  bhp: read in the file of form: mos1S002-obj.pi
    !       header was read above by read_header_info
    !       kept opened above, just pass inSPEC_T in/out
    !       to retrieve actual spectrum 
    ! old/new obj_cnts/specobs stat/specobs_U
    ! OLD: obj_cnts NEW: specO_JP
    ! OLD: stat     NEW: specOU_RP
    ! these files have 8-bin unsigned integer (I not J) values
    !
    call read_pha(inSPEC_T,chansO,1,chansO_IP,specO_JP,dummy_RP,specOU_RP)
    call release(inSPEC_T) 
    call release(inSPEC_S) ! done with observation spec file
    !
    ! chansO = number of channels in observation spectrum
    !
    !  Get the channel to energy conversion
    !  read the rmf file, e.g.: mos1S002.rmf
    !  response matrix file has 2 extensions MATRIX and EBOUNDS.
    !  old subsn *read* MATRIX, but never uses it. 
    !  old call :
    !  call read_rmf (rmf_obj_F,2400,detm,chan,eminin,emaxin,nchanrin, &
    !     emin,emax,nchanrout)
    !
    status = 0
    warndie = .TRUE. ! die if not found or unreadable
    call check_open_set(inRMF_F,inRMF_S,1,status,warndie)
    allocate(emin(chansO),emax(chansO),ener(chansO))
    emin = 0.0
    emax = 0.0
    ener = 0.0
    !
    call read_rmf(inRMF_S,chansO,rmfChans,emin,emax,matrix)
    call release(inRMF_S) ! done with rmf file
    write(errstr,*)"Channel converted to energy with RMF"
    call message(AppMsg,Noisy)
    do ichan=1,chansO-1
       ener(ichan) = (emin(ichan) + emax(ichan))/2.
       write(errstr,*)ichan,' ',ener(ichan)
       call message(AppMsg,Noisy)
    enddo
    !
    do i=1,chansO-1
       !
       !  Set channel ranges for some energy bands
       !
       if((ener(i) < 0.3) .and. (ener(i+1) >= 0.3)) &
            plc_al(1) = i + 1
       if((ener(i) .le. 10.0) .and. (ener(i+1) > 10.0)) &
            plc_al(2) = i
       if((ener(i) < 0.4) .and. (ener(i+1) >= 0.4)) &
            plc_lo(1) = i + 1
       if((ener(i) < 0.8) .and. (ener(i+1) >= 0.8)) &
            plc_lo(2) = i
       if((ener(i) < 2.5) .and. (ener(i+1) >= 2.5)) &
            plc_hi(1) = i + 1
       if((ener(i) < 5.0) .and. (ener(i+1) >= 5.0)) &
            plc_hi(2) = i

       !  Set channel ranges for the gold (Au) contamination

       if((ener(i) < 2.04) .and. (ener(i+1) >= 2.04)) &
            plc_aua(1) = i + 1
       if((ener(i) <= 2.33) .and. (ener(i+1) > 2.33)) &
            plc_aua(2) = i
       if((ener(i) < 9.38) .and. (ener(i+1) >= 9.38)) & 
            plc_aub(1) = i + 1
       if((ener(i) <= 9.98) .and. (ener(i+1) > 9.98)) &
            plc_aub(2) = i
       if((ener(i) < 11.03) .and. (ener(i+1) >= 11.03)) &
            plc_auc(1) = i + 1
       if((ener(i) <= 11.82) .and. (ener(i+1) > 11.82)) &
            plc_auc(2) = i

       !  Set channel ranges for the Al and Si Kalpha contamination

       if((ener(i) < 1.2) .and. (ener(i+1) >= 1.2)) &
            plc_alsi(1) = i + 1
       if((ener(i) <= 1.9) .and. (ener(i+1) > 1.9)) &
            plc_alsi(2) = i

       !  Set channel ranges for the energy band

       if((bandl == 0.0) .and. (bandh == 0.0)) then
          if((ener(i) < 0.3) .and. (ener(i+1) >= 0.3)) &
               band(1) = i + 1
          if((ener(i) <= 8.0) .and. (ener(i+1) > 8.0)) &
               band(2) = i
       else
          if((ener(i) < bandl) .and. (ener(i+1) >= bandl)) &
               band(1) = i + 1
          if((ener(i) <= bandh) .and. (ener(i+1) > bandh)) &
               band(2) = i
       endif

       !  Set channel ranges for the out

       if((ener(i) < 1.0) .and. (ener(i+1) >= 1.0)) &
            out(1) = i + 1
       if((ener(i) <= 2.5) .and. (ener(i+1) > 2.5)) &
            out(2) = i
    enddo
    !
    ! SS: Read the observation chip spectra to get the exposure
    ! bhp: done above already
    ! The obs data are the observation data
    !
    ! bhp: Individual CCD data for the object region will have suffix "ccdo",
    ! e.g. areaccdo
    !
    write(errstr,*)'Processing the observation data for each CCD'
    call message(AppMsg,Always) ! old diag handled by SAS_VERBOSITY now
    write(tstring,4000) " Bins   Expose   Back   Area   Rev  Stat'"
4000 format(1x,A)
    !
    ! bhp: read in each chip spectral file (e.g. mos1S002-7obj.pi)
    !      loop through all the object by ccd spectra
    !
    allocate(expoobs_tot(7),areaobs_tot(7))
    expoobs_tot = 0.   ! OLD: reg_exp_obs
    areaobs_tot = 0.   ! OLD: reg_area_obs
    !
    write(errstr,*)trim(adjustl(tstring))
    call message(AppMsg,Verbose) 
    !
    do i=1,7
       if( ccds(i) ) then ! bhp: boolean now
          write(str_ccd,'(I1)')i
          ! OLD: mos1S002-3obj.pi NEW: mos1S002-fovccd3.pi
          specccdo_F = trim(adjustl(prefix)) // "-" // &
              "fovccd" // trim(adjustl(str_ccd)) // ".pi"
          write(errstr,*)"Current ccd spec file: ",trim(specccdo_F)
          call message(AppMsg,Noisy)
          revlccdo = 0
          backccdo = 0.
          areaccdo = 0.
          expoccdo = 0.
          status = 0
          warndie = .FALSE. ! it's ok if not there
          call check_open_set(specccdo_F,specccdo_S,1,status,warndie)
          ! only getting expo/area data from these spec files
          if (status == 1) then ! found and readable
             typeF = 2
             call read_header_info(typeF,specccdo_S,out_T,out_A,instccdo,obsidccdo,&
                  expidccdo,smodeccdo,frmtimeccdo,filterccdo,revlccdo,backccdo,&
                  areaccdo,expoccdo,elowccdo,ehighccdo,chansccdo,warndie)
             call release(specccdo_S)
             if (revlccdo /= revlO) then ! one sanity check, add others?
                call error("wrongOrbit",&
                     "By-ccd spectra not from same revolut as obs spectra!")
             endif
          else
             write(title,*)"NOspecCCD",str_ccd
             call warning(trim(adjustl(title)),"Spectral file missing for this CCD.")
             call warning("checkMOSSPECTRA","CCD may not have been processed in mosspectra.")
             expoccdo = 0.
             backccdo = 0.
          endif
!          write(errstr,*)'YYYYY i, chansO,expoccdo,backccdo/amsca,areaccdo,revlccdo,status'
!          call message(AppMsg,Always) ! change to Verbose???!!!
!          write(errstr,4005)i,chansO,expoccdo,backccdo/amsca,areaccdo,revlccdo,status
!4005      format(1x,i2,1x,i5,1x,f9.1,2f7.1,2i6)
          call message(AppMsg,Always) ! change to Verbose???!!!
          ! fill exposure/area array for each selected ccd
          ! old/new reg_exp_obs/expoobs_tot reg_area_obs/areaobs_tot
          expoobs_tot(i) = expoccdo  ! will stay zero if no file
          areaobs_tot(i) = backccdo  ! why back and not AREA???!!!
       endif

    enddo
    write(errstr,*)'Exposure, area/amsca from the observation data:'
    call message(AppMsg,Always)
    do i=1,7
       if(ccds(i)) then ! BHP: a boolean now
          write(errstr,4010) i, expoobs_tot(i), &
               areaobs_tot(i)/amsca
 4010      format(1x,'Chip ',i1,'  exposure =',f10.1, &
               '  backscale =',2f7.1)
          call message(AppMsg,Always) ! bhp: verbose maybe???
       endif
    enddo
    !
    ! SS: Read the fwc spectra from the object region
    ! Figure out which chips are involved
    ! Use areafwco_tot as a vector describing which chips are in use
    !
    ! The ff data are the FWC data from the object region
    ! OLD: mos1S001-2ff.pi NEW: mos1S001-fovfwcccd2.pi
    ! a 2400 channel spectra
    !
    ! FWC data for the object region will have prefix "fwc",
    ! e.g. fwco (obs) fwcc (corner)
    !
    write(errstr,*)'Process the FWC data from the object region.'
    call message(AppMsg,Always)
    !
    allocate(expofwco_tot(7),areafwco_tot(7))
    allocate(specfwco(chansO),specfwco_U(chansO))
    allocate(specfwco_tot(chansO,7),specfwco_tot_U(chansO,7))
    expofwco_tot = 0;
    areafwco_tot = 0;
    specfwco_tot = 0;
    specfwco_tot_U = 0;
    do i=1,7
       if( ccds(i) ) then ! bhp: boolean now
          ! SPECTRUM (channel/counts x 2400 + GTI/REGs)
          write(str_ccd,'(I1)')i
          ! OLD: mos1S002-3ff.pi NEW: mos1S002-fovfwcccd3.pi
          specfwco_F = trim(adjustl(prefix)) // "-" // &
              "fovfwcccd" // trim(adjustl(str_ccd)) // ".pi"
          write(errstr,*)"Current FWC FOV spec file: ",trim(specfwco_F)
          call message(AppMsg,Noisy)
          status = 0
          warndie = .FALSE. ! bhp: it's ok if not there
          ! zero all of these for read_pha_header?
          call check_open_set(specfwco_F,specfwco_S,1,status,warndie) ! check this???!!!
          revlfwco = 0 
          backfwco = 0. 
          areafwco = 0.
          expofwco = 0.
          specfwco = 0.
          specfwco_U = 0.
          if (status .eq. 1) then ! found and readable
             typeF = 2 ! type2 = spectral file
             ! most will not be in FWC data!!!???
             call read_header_info(typeF,specfwco_S,out_T,out_A,instfwco,obsidfwco,&
                  expidfwco,smodefwco,frmtimefwco,filterfwco,revlfwco,backfwco,areafwco,&
                  expofwco,elowfwco,ehighfwco,chansfwco,warndie)
             call read_pha(out_T,chansfwco,1,chansfwco_IP,specfwco_JP,dummy_RP,specfwcoU_RP)
             call release(out_T)
             call release(specfwco_S)
          else
             write(title,*)"NOspecCCD",str_ccd
             call warning(trim(adjustl(title)),"FWC spectral file missing for selected CCD.")
             expofwco = 0.
             areafwco = 0.
          endif
          ! OLD/NEW: reg_exp_fwc/expofwco_tot expo/expofwco
          expofwco_tot(i) = expofwco ! 0 if ff file not exist
          areafwco_tot(i) = areafwco ! i.e. if status ne 1
          !
          if((backfwco /= 0) .and. (expofwco /= 0)) then
             ! There is active area on this chip, normalize the spectrum
             ! if this is NOT true, values stay as is (i.e. 0)
             do ichan=1,chansfwco ! or chansO???!!!
                ! old/new reg_spec/specfwco_tot reg_uncr/specfwco_tot_U
                ! old/new spec#1/specfwco uncr#1/specfwco_U
                specfwco_tot(ichan,i) = real(specfwco_JP(ichan))/backfwco/expofwco
                specfwco_tot_U(ichan,i) = specfwcoU_RP(ichan)/backfwco/expofwco
             enddo
             !old/new reg_area/areafwco_tot why back not areafwco???
             areafwco_tot(i) = backfwco 
          else
             do ichan=1,chansfwco
                specfwco_tot(ichan,i) = 0.0
                specfwco_tot_U(ichan,i) = 0.0
             enddo
          end if
       endif
       if (status .ne. 1) then
          write(errstr,*)'CCD ',i,' not processed ',status
          call message(AppMsg,Always) ! change this???!!!
       endif
       !          write(errstr,4005)i,chansO,expofwco,backfwco/amsca, &
       !               areafwco,revlfwco,status
       !          call message(AppMsg,Always) ! change this???!!!
    enddo
    !
    write(errstr,*)"Backscale and exposure from the FWC data:"
    call message(AppMsg,Always)
    do i=1,7
       if( ccds(i) ) then ! bhp: boolean now
          write(errstr,4010) i,expofwco_tot(i),areafwco_tot(i)/amsca
          call message(AppMsg,Always) ! bhp: verbose????
       endif
    enddo
    !    
    ! Remove any chip that has less area than the limiting area, lim_area
    ! bhp: why is lim_area hard coded to 0.0???!!!
    !
    do i=1,7
       if( ccds(i) ) then ! bhp: boolean now
          if((areafwco_tot(i) .lt. lim_area) .or. &               
               (areafwco_tot(i) == 0.0) .or. &
               (areaobs_tot(i) == 0.0)) then
             areafwco_tot(i) = 0.0
             do ichan=1,chansfwco
                specfwco_tot(ichan,i) = 0.0
                specfwco_tot_U(ichan,i) = 0.0
             enddo
          endif
       endif
    enddo
    !
    !  Read corner spectra from the observation
    !  First set up the spectrum masks
    !
    !  Testing here to see if we really do need to exclude the corner 
    !  data from chips 2,7 because of Au issues
    !
    ! corner data from the observation for each have suffix "coro/corc"

    allocate(maskcoro_tot(chansO,7),maskcorc_tot(chansO,7))
    ! old/new oc_mask/maskcoro_tot
    do i=1,7
       if( ccds(i) ) then ! boolean
          do ichan=1,chansO
             maskcoro_tot(ichan,i) = 1.
          enddo
       endif
    enddo
    !
    !  Even if one of the outer chips is not being used
    !  we still need the corner data if chip 1 is on
    !
    !  The oc data are observation data from the corners
    ! OLD: mos1S001-2oc.pi NEW: mos1S001-coroccd2.pi
    !
    !    write(errstr,4000) ! bhp: see 4000 stmt ~line 380
    !    call message(AppMsg,Always)
    !
    !    write(errstr,*)trim(tstring) ! title: bins expo area etc
    !    call message(AppMsg,Always) ! change this?
    !
    ! *** corner spectral file = old db_file ***
    ! next line opens the db_file fits file
    ! open/read the "CALDB" mosN-qpb.fits file ONCE, then
    ! loop through relevant extensions
    !
    ! open QPB testprods file now that it's needed
    !
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(QPB_F),QPB_S,1,status,warndie)
    ! Set up to process corner data
    allocate(speccoro(chansO),speccoro_U(chansO))
    allocate(aug_spec(chansO))
    allocate(expocoro_tot(7),areacoro_tot(7))
    allocate(speccoro_tot(chansO,7),speccoro_tot_U(chansO,7))
    did = .FALSE.
    expocoro_tot = 0.   ! 7 element array
    areacoro_tot = 0.   ! 7 element array
    speccoro_tot = 0.   ! 2400x7 element array
    speccoro_tot_U = 0. ! 2400x7 element array
    !
    write(errstr,*)'Process the observation data from the corner regions'
    call message(AppMsg,Always)
    do i=2,7
       speccoro = 0.
       speccoro_U = 0.
       if( ccds(i) ) then ! boolean for each ccd chosen
          ! SPECTRUM (channel/counts x 2400 + GTI/REGs)
          write(str_ccd,'(I1)')i
          ! OLD: mos1S002-3oc.pi NEW: mos1S002-coroccd3.pi
          speccoro_F = trim(adjustl(prefix)) // "-" // &
               "corccd" // trim(adjustl(str_ccd)) // ".pi"
          write(errstr,*)"Current obs corner spec file: ",trim(speccoro_F)
          call message(AppMsg,Noisy) ! change later ???!!!
          status = 0
          warndie = .FALSE. ! BHP: it's ok if not there
          call check_open_set(speccoro_F,speccoro_S,1,status,warndie) 
          rate = 0. ! only needed
          hard = 0. ! for augmented
          high = 0. ! spectrum 
          low = 0.  ! calculation
          expocoro = 0.0
          areacoro = 0.0
          revlcoro = 0
          speccoro = 0.
          speccoro_U = 0.
          if (status == 1) then ! found and readable
             warndie = .TRUE.   ! die if cor file doesn't "work"
             call read_header_info(typeF,speccoro_S,out_T,out_A,instcoro,obsidcoro,&
                  expidcoro,smodecoro,frmtimecoro,filtercoro,revlcoro,backcoro,areacoro,&
                  expocoro,elowcoro,ehighcoro,chanscoro,warndie)
             ! old/new spec#2/speccoro uncr#2/speccoro_U
             ! OLD: spec NEW: speccoro_JP OLD: uncr NEW: speccoroU_RP
             call read_pha(out_T,chanscoro,1,chanscoro_IP,speccoro_JP,dummy_RP,speccoroU_RP)
             ! speccoro_JP is an Int32 pointer returned from reading the spectrum
             ! math will be done with reals below
             if (revlcoro /= revlO) then
                call error("wrongOrbit",&
                     "By-ccd Corner spectra not from same orbit as obs spectra!")
             endif
             ! old/new oc_exp_obs/expocoro_tot
             expocoro_tot(i) = expocoro
             ! takes place of old print statement:
             write(errstr,4006) chanscoro,expocoro,backcoro,areacoro,revlcoro
4006         format(1x,i5,1x,f9.1,1x,2f12.1,1x,i6)
             call message(AppMsg,Always)
             ! old mos_back: rate = rate + spec(ichan)
             ! old/new rate/ratecoro spec#2/speccoro
             do ichan=plc_al(1),plc_al(2)
                rate = rate + real(speccoro_JP(ichan))
             enddo
             rate = rate / expocoro
             !
             !  Scale the MOS1 CCD#4 count rate if needed
             !
             if((mos1) .and. (i == 4) .and. &
                  (revlO >= 2383)) then
                rmocftb = 32.2
                rmocfob = backcoro / amsca
                rmocfori = rate
                rate = rate * rmocftb / rmocfob
             endif
             !
             do ichan=plc_hi(1),plc_hi(2)
                high = high + real(speccoro_JP(ichan))
             enddo
             do ichan=plc_lo(1),plc_lo(2)
                low = low + real(speccoro_JP(ichan))
             enddo
             hard = high / low
             !
             ! Augment the spectrum
             ! aug_get_exp is now a sub inside goflib/esas_mod.f90
             ! old/new c_val(1)/rate c_val(2)/hard c_val(3)/revlO
             aug_spec = 0.
             aug_expo = 0. ! fix this ???!!!

             call augment_mos(outQDP_F,QPB_S,rate,hard,revlO,aug_scal,aug_time,&
                  withqdp,aug_spec,aug_expo,mosnum,i,chansO,obsidO,expidO)
             write(errstr,*)'Augmented exposure = ',aug_expo
             call message(AppMsg,Always)
             if((mos1) .and. (i == 4) .and. &
                  (revlO >= 2383)) then
                rate = rmocfori
             endif
             do ichan=0,chanscoro-1
                speccoro(ichan+1) = speccoro_JP(ichan) + aug_spec(ichan+1)
!                print*,speccoro(ichan),speccoro_JP(ichan),aug_spec(ichan)
             enddo
             expocoro = expocoro + aug_expo
             aug_rate = 0.0

             do ichan=plc_al(1),plc_al(2)
                aug_rate = aug_rate + speccoro(ichan)
             enddo

             aug_rate = aug_rate / expocoro ! chansO should == chanscoro, right???!!!
             do ichan=1,chanscoro ! ???!!! chansO all the way???
                speccoro(ichan) = speccoro(ichan) * rate / aug_rate
             enddo
             !
             !  Normalize the spectrum
             ! old/new spec/speccoro uncr/speccoro_U
             ! old/new oc_spec/speccoro_tot oc_uncr/speccoro_tot_U
             do ichan=1,chansO
                speccoro_U(ichan) = sqrt(speccoro(ichan))
                speccoro_tot(ichan,i) = speccoro(ichan)/backcoro/expocoro
                speccoro_tot_U(ichan,i) = speccoro_U(ichan)/backcoro/expocoro
             enddo
             write(errstr,2525) "Corner ccd/back/expo = ",i, backcoro, expocoro
             call message(AppMsg,Always)
2525         format(A,i2,2(1x,f12.2))
             ! old/new oc_area/areacoro_tot reg_exp/expocoro_tot
             areacoro_tot(i) = backcoro 
             expocoro_tot(i) = expocoro
             call release(out_T)
             call release(speccoro_S)
          else ! if not found nor readable, set things to 0
             write(errstr,*)"No corner data for CCD: ",i
             call message(AppMsg,Noisy)
             if(areafwco_tot(i) /=  0.0) then ! OLD: reg_area
                do ichan=1,chansO
                   specfwco_tot(ichan,i) = 0.0      ! OLD: reg_spec
                   specfwco_tot_U(ichan,i) = 0.0    ! OLD: reg_uncr
                enddo
                areafwco_tot(i) = 0.0           ! OLD: reg_area
                !
                !  Will not kill but will not return, we will try to recover
                !
             endif
          endif
!          write(errstr,4005)i,chansO,expocoro,backcoro/amsca,&
!               areacoro,revlcoro,status
!          call message(AppMsg,Noisy)
       endif
    enddo
    call release(QPB_S)
    write(errstr,*)'Backscale and augmented exposure from the ', &
         'corners of the observation data'
    call message(AppMsg,Always)

    do i=2,7
       if( ccds(i) ) then ! boolean
          write(errstr,4010) i, expocoro_tot(i), areacoro_tot(i)/amsca 
          call message(AppMsg,Always)
       endif
    enddo
    !
    !  Do the center chip if necessary
    !
    allocate(oc1(chansO),uc1(chansO),mk1(chansO))
    oc1 = 0.
    mk1 = 0.
    if((areafwco_tot(1) /= 0.0) .and. ( ccds(1) )) then ! bhp: boolean now

       !  The center chip is active
       !  create a mask for normalizing the center chips spectrum
       !  determine how many similar chips have data

       write(errstr,*)'Process the observation data from the center chip'
       call message(AppMsg,Always)

       n_real = n_sims
       do i=1,n_sims
          if(areacoro_tot(sim_chips(i)) == 0.0) then
             n_real = n_real - 1
             sim_chips(i) = 0
          endif
       enddo
       !
       if(n_real == 0) then
          !  This is really an error from which we can not recover
          write(errstr,*) "No corner data for chip 1 model"
          call error("NoCorDataCCD1model",trim(errstr))
       else
          n_sims = n_real
          it = 0
          do i=1,4
             if(sim_chips(i) .ne. 0) then
                it = it + 1
                sim_chips(it) = sim_chips(i)
             endif
          enddo
       endif
       write(errstr,*)"Number of SIM chips ", n_sims
       call message(AppMsg,Always)
       write(errstr,*)"Chips: ",(sim_chips(i),i=1,n_sims)
       call message(AppMsg,Always)

       !  Sum over the similar chips

       do i=1,n_sims
          do ichan=1,chansO
             oc1(ichan) = oc1(ichan) + &
                  speccoro_tot(ichan,sim_chips(i))* &
                  maskcoro_tot(ichan,sim_chips(i))
             uc1(ichan) = uc1(ichan) + &
                  (speccoro_tot_U(ichan,sim_chips(i))**2)* &
                  maskcoro_tot(ichan,sim_chips(i))
             mk1(ichan) = mk1(ichan) + &
                  maskcoro_tot(ichan,sim_chips(i))
          enddo
       enddo
       !
       ! Check to see whether there are any data
       !
       if ( any(mk1 .ne. 0) ) then ! bhp: use any statement 
          write(errstr,*) "Found data in chip 1"
          call message(AppMsg,Noisy)
       else
          call error ("NoDataChip1",&
               "No data to create a spectrum for chip 1")
       end if

       do ichan=1,chansO
          if(mk1(ichan) /= 0.0) then
             oc1(ichan) = oc1(ichan)/mk1(ichan)
             uc1(ichan) = sqrt(uc1(ichan))/mk1(ichan)
          else
             oc1(ichan) = 0.0
             uc1(ichan) = 0.0
          endif
          speccoro_tot(ichan,1)   = oc1(ichan)
          speccoro_tot_U(ichan,1) = uc1(ichan)
          write(80,*)ichan,mk1(ichan),oc1(ichan),uc1(ichan)
       enddo
       do i=1,7
          if(ccds(i)) then ! bhp: boolean now
             tot = 0.0
             do ichan=band(1),band(2)
                tot = tot + speccoro_tot(ichan,i)
                write(81,*)i,band(1),band(2),speccoro_tot(ichan,i),tot
             enddo
             write(errstr,*)'Chip',i,' observation corner total: ', tot
             call message(AppMsg,Always)
          endif
       enddo
    endif
    
    !  Read FWC corner spectra

    !  The fc data are the FWC data from the corners

    write(errstr,*)'Process the FWC corner data'
    call message(AppMsg,Always)

    write(errstr,4000) ! bhp: from old 'print 4000' statement
    call message(AppMsg,Verbose)

    allocate(specfwcc_tot(chansO,7),specfwcc_tot_U(chansO,7))
    allocate(specfwcc(chansO),specfwcc_U(chansO),fc1(chansO))
    allocate(areafwcc_tot(7),expofwcc_tot(7))
    areafwcc_tot = 0.
    expofwcc_tot = 0.
    specfwcc_tot = 0.
    specfwcc_tot_U = 0.
    !
    do i=2,7
       specfwcc = 0.
       specfwcc_U = 0.
       if( ccds(i) ) then ! bhp: boolean now
          ! SPECTRUM (channel/counts x 2400 + GTI/REGs)
          write(str_ccd,'(I1)')i
          ! OLD: mos1S002-3fc.pi NEW: mos1S002-corfwcccd3.pi
          specfwcc_F = trim(adjustl(prefix)) // "-" // &
               "corfwcccd" // trim(adjustl(str_ccd)) // ".pi"
          write(errstr,*)"Current FWC corner spec file: ",trim(speccoro_F)
          call message(AppMsg,Always) ! change later ???!!!
          status = 0
          warndie = .FALSE. ! bhp: it's ok if not there
          call check_open_set(specfwcc_F,specfwcc_S,1,status,warndie) 
          fc1 = 0.
          areafwcc = 0.
          expofwcc = 0.
          backfwcc = 0.
          specfwcc = 0.
          specfwcc_U = 0.
          if (status == 1) then ! found and readable
             warndie = .TRUE.   ! die if cor file doesn't "work" ???!!! change this???
             call read_header_info(typeF,specfwcc_S,out_T,out_A,instfwcc,obsidfwcc,&
                  expidfwcc,smodefwcc,frmtimefwcc,filterfwcc,revlfwcc,backfwcc,areafwcc,&
                  expofwcc,elowfwcc,ehighfwcc,chansfwcc,warndie)
             !
             ! old/new spec#2/specfwcc uncr#2/specfwcc_U
             ! OLD: spec NEW: specfwcc_JP OLD: uncr NEW: specfwccU_RP             
             !
             call read_pha(out_T,chansfwcc,1,chansfwcc_IP,specfwcc_JP,dummy_RP,specfwccU_RP)
             ! specfwcc_JP is an Int32 pointer returned from reading the spectrum
             ! math will be done with reals below

             !  Normalize the spectrum
             ! old/new fc_spec/specfwcc_tot spec/specfwcc
             ! old/new fc_uncr/specfwcc_tot_U uncr/specfwcc_U
             do ichan=1,chansO
                specfwcc_tot(ichan,i) = specfwcc_JP(ichan-1)/backfwcc/expofwcc
                specfwcc_tot_U(ichan,i) = specfwccU_RP(ichan-1)/backfwcc/expofwcc
             enddo
             ! old/new fc_area/areafwcc_tot back/areafwcc
             ! old/nre fc_exp/expofwcc_tot  expo/expofwcc
             areafwcc_tot(i) = backfwcc
             expofwcc_tot(i) = expofwcc
             call release(out_T)
             call release(specfwcc_S)
          endif
       endif
    enddo
    write(errstr,*) &
         "Backscale and exposure from the corners of the FWC data"
    call message(AppMsg,Always)

    do i=2,7
       if( ccds(i) ) then ! bhp: boolean now
          write(errstr,4010) i, expofwcc_tot(i), areafwcc_tot(i)/amsca
          call message(AppMsg,Always)
       endif
    enddo

    !
    !  Create a corner spectrum for the center chip
    !
    if( ccds(1) ) then ! bhp: boolean now
       do i=1,n_sims
          do ichan=1,chansO
             fc1(ichan) = fc1(ichan) + &
                  specfwcc_tot(ichan,sim_chips(i))* &
                  maskcoro_tot(ichan,sim_chips(i))
          enddo
       enddo
       t = 0.0
       do ichan=1,chansO
          if(mk1(ichan) /= 0) then
             fc1(ichan) = fc1(ichan)/mk1(ichan)
          endif
          specfwcc_tot(ichan,1) = fc1(ichan)
          t = t + specfwcc_tot(ichan,1)
       enddo
       if(t == 0.0) then
          write(errstr,*)"No counts for the pseudo CCD1 corner spec"
          call error("NoPseudoCCD1cnts",trim(errstr))
       endif
    endif
    !
    ! Create the ratio of region/corner for FWC data
    !
    ! old/new reg_rati/ratio_tot reg_mask/mask_tot
    !
    allocate(ratio_tot(chansO,7),mask_tot(chansO,7))
    allocate(temp_top(chansO),temp_bot(chansO))
    ratio_tot = 0.
    mask_tot = 0.
    temp_top = 0.
    temp_bot = 0.
    !
    do i=2,7
       if((areafwco_tot(i) /= 0.0) .and. (ccds(i))) then
          do ichan=40,chansO
             i1 = ichan - smo_fact
             if(i1 < 40) i1 = 40
             i2 = ichan + smo_fact
             if(i2 > chansO) i2 = chansO
             t1 = 0.0
             t2 = 0.0
             it = 0
             do ii=i1,i2
                t1 = t1 + specfwco_tot(ii,i)
                t2 = t2 + specfwcc_tot(ii,i)
                it = it + 1
             enddo
             if(it > 0) then
                temp_top(ichan) = t1/it
                temp_bot(ichan) = t2/it
             else
                temp_top(ichan) = 0.0
                temp_bot(ichan) = 0.0
             endif
             !print*,chansO,i1,i2,t1,t2,it,temp_top(ichan),temp_bot(ichan)
          enddo

          ! old/new reg_rati/ratio_tot reg_mask/mask_tot
          do ichan=1,chansO
             if(temp_bot(ichan) /= 0) then
                mask_tot(ichan,i) = 1.0
                ratio_tot(ichan,i) = &
                     temp_top(ichan)/temp_bot(ichan)
             else
                mask_tot(ichan,i) = 0.0
                ratio_tot(ichan,i) = 0.0
             endif
          enddo
       endif

       do ichan=1,chansO
          write(66,*)i,ichan,temp_top(ichan),temp_bot(ichan)
       end do
    enddo

    !
    !  Single out chip 1 for special treatment again
    !
    if((areafwco_tot(1) /= 0.0) .and. (ccds(1)) ) then ! bhp: boolean now
 
       !  Does the spectrum have gold gaps?

       itest = 1
       ites = 0
       do ichan=plc_al(1),plc_al(2)
          if(speccoro_tot(ichan,1) == 0.0) then
             itest = 0
             ites = ites + 1
          endif
       enddo
       if((itest == 1) .or. (ites < 10)) then
          do ichan=40,chansO
             i1 = ichan - smo_fact
             if(i1 < 40) i1 = 40
             i2 = ichan + smo_fact
             if(i2 > chansO) i2 = chansO
             t1 = 0.0
             t2 = 0.0
             it = 0
             do ii=i1,i2
                t1 = t1 + specfwco_tot(ii,1)
                t2 = t2 + specfwcc_tot(ii,1)
                it = it + 1
             enddo
             if(it > 0) then
                temp_top(ichan) = t1/it
                temp_bot(ichan) = t2/it
             else
                temp_top(ichan) = 0.0
                temp_bot(ichan) = 0.0
             endif
          enddo
          do ichan=1,chansO
             if(temp_bot(ichan) .ne. 0) then
                mask_tot(ichan,1) = 1.0
                ratio_tot(ichan,1) = &
                     temp_top(ichan)/temp_bot(ichan)
             else
                mask_tot(ichan,1) = 0.0
                ratio_tot(ichan,1) = 0.0
             endif
          enddo
       else
          write(errstr,2994) (speccoro_tot(ichan,1),ichan=1,chansO)
2994      format(8(e10.3))
          call message(AppMsg,Always)
          write(errstr,*)itest, ites, plc_al(1), plc_al(2)
          call message(AppMsg,Always)
          write(errstr,*)&
               "Chip 1 background has gold gaps - no CCD#3 or CCD#7 data"
          call error("GoldGaps",trim(errstr))              
       endif
    endif
       !  Print out some diagnostic information
    write(errstr,*)'Backscale for the FWC corner ', &
         'and FWC region data'
    call message(AppMsg,Always)
    do i=1,7
       write(errstr,4015) i, areafwcc_tot(i)/amsca,  &
            areafwco_tot(i)/amsca
4015   format(1x,'Chip ',i1,' backscale =',f7.1, &
            ' backscale =',f7.1)
       call message(AppMsg,Always)
    end do

    !
    t1 = 0.0
    t2 = 0.0
    do i=1,7
       t1 = t1 + areafwcc_tot(i)
       t2 = t2 + areafwco_tot(i)
    enddo
    write(errstr,*)'FC_AREA, REG_AREA, RATIO: ', t1, t2, t1/(t1 + t2)
    call message(AppMsg,Always)
    !
    !  Create the masks to remove strong instrumental lines
    !  Remove the Al-Si line from all spectra
    !
    do ichan=plc_alsi(1),plc_alsi(2)
       do i=1,7
          maskcoro_tot(ichan,i) = 0
       enddo
    enddo
    !
    !  Sum the background over all active chips and make an image
    !
    if(withqdp) then ! bhp: added booleanParameter, was if diag > 2
       qdpratio_F = trim(prefix) // '-ratio-indiv.qdp' !86
       ! old/new 86/lunratio
       call open_ascii_file(qdpratio_F,lunratio)
       write(errstr,*)'LUN for QDP ASCII file',trim(adjustl(qdpratio_F)),' = ',lunratio
       call message(AppMsg,Noisy) ! change to Noisy!!!???
       write(lunratio,*) 'viewport 0.2 0.2 0.95 0.9'
       write(lunratio,*) 'font roman'
       write(lunratio,*) 'cs 1.5'
       write(lunratio,*) 'log y'
       write(lunratio,*) 'r y 1.0 1.e8'
       write(lunratio,*) 'la y Ratio'
       write(lunratio,*) 'la x Channel'
       !
       qdpaug_F = trim(prefix) // '-aug-indiv.qdp'     !87
       ! old/new 87/lunaug
       call open_ascii_file(qdpaug_F,lunaug)
       write(errstr,*)'LUN for QDP ASCII file',trim(adjustl(qdpaug_F)),' = ',lunaug
       call message(AppMsg,Noisy) ! change to Noisy!!!???
       write(lunaug,*) 'viewport 0.2 0.2 0.95 0.9'
       write(lunaug,*) 'read serr 2'
       write(lunaug,*) 'font roman'
       write(lunaug,*) 'cs 1.5'
       write(lunaug,*) 'log y'
       write(lunaug,*) 'r y 1.e-2 1.e6'
       write(lunaug,*) 'la y Scaled Counts'
       write(lunaug,*) 'la x Channel'
       !
       qdpbacki_F = trim(prefix) // '-back-indiv.qdp'  !88
       ! old/new 88/lunbacki
       call open_ascii_file(qdpbacki_F,lunbacki)
       write(errstr,*)'LUN for QDP ASCII file',trim(adjustl(qdpbacki_F)),' = ',lunbacki
       call message(AppMsg,Noisy) ! change to Noisy!!!???
       write(lunbacki,*) 'viewport 0.2 0.2 0.95 0.9'
       write(lunbacki,*) 'read serr 2'
       write(lunbacki,*) 'font roman'
       write(lunbacki,*) 'cs 1.5'
       write(lunbacki,*) 'log y'
       write(lunbacki,*) 'r y 0.005 0.5e7'
       write(lunbacki,*) 'la y Scaled Counts'
       write(lunbacki,*) 'la x Channel'

       qdpbacka_F = trim(prefix) // '-back-accum.qdp'  !89
       call open_ascii_file(qdpbacka_F,lunbacka)
       write(errstr,*)'LUN for QDP ASCII file',trim(adjustl(qdpbacka_F)),' = ',lunbacka
       call message(AppMsg,Noisy) ! change to Noisy!!!???
       ! old/new 89/lunbacka
       write(lunbacka,*) 'viewport 0.2 0.2 0.95 0.9'
       write(lunbacka,*) 'read serr 2'
       write(lunbacka,*) 'font roman'
       write(lunbacka,*) 'cs 1.5'
       write(lunbacka,*) 'log y'
       write(lunbacka,*) 'r y 1.e-2 1.e6'
       write(lunbacka,*) 'la y Scaled Counts'
       write(lunbacka,*) 'la x Channel'

    endif
    allocate(specout(chansO),specout_U(chansO),maskout(chansO))

    maskout = 0.
    specout = 0.
    specout_U = 0.

    do i=1,7
       write(str_ccd,'(I1)')i       
       if((areafwco_tot(i) /= 0) .and. (ccds(i))) then
          write(errstr,*)&
               "Add in CCD#"//trim(adjustl(str_ccd))//" background spectrum"
          call message(AppMsg,Always)
          ! old/new reg_mask/mask_tot spec#3/specout uncr#3/specout_U
          ! oc_mask/maskcoro_tot oc_spec/speccoro_tot reg_area/areafwco_tot
          ! mask/maskout
          do ichan=1,chansO
             if((mask_tot(ichan,i) /= 0.0) .and. &
                  (maskcoro_tot(ichan,i) /= 0.0)) then
                specout(ichan) = specout(ichan) + speccoro_tot(ichan,i)* &
                     ratio_tot(ichan,i)*areafwco_tot(i)
                specout_U(ichan) = &
                     specout_U(ichan) + (speccoro_tot_U(ichan,i) * &
                     ratio_tot(ichan,i)*areafwco_tot(i))**2
                maskout(ichan) = maskout(ichan) + areafwco_tot(i)
             endif ! bhp: boolean now instead of diag > 2
             if((withqdp) .and. (maskout(ichan) > 0.0)) then
                write(lunratio,*) ichan, &
                     (10.0**i)*ratio_tot(ichan,i)
                write(lunaug,*) ichan, &
                     1.e10*(10.0**i)*speccoro_tot(ichan,i), &
                     1.e10*(10.0**i)*speccoro_tot_U(ichan,i)
                write(lunbacki,*) ichan, &
                     1.e3*(10.0**i)*speccoro_tot(ichan,i)* &
                     ratio_tot(ichan,i)*areafwco_tot(i), &
                     1.e3*(10.0**i)*sqrt((speccoro_tot_U(ichan,i)* &
                     ratio_tot(ichan,i)*areafwco_tot(i))**2)
                write(lunbacka,*) ichan, &
                     1.e10*(10.0**i)* &
                     specout(ichan)/maskout(ichan), &
                     1.e10*(10.0**i)* &
                     sqrt(specout_U(ichan))/maskout(ichan)
             endif
          enddo
          tmodcounts = 0.0
          tot1 = 0.0
          tot2 = 0.0
          do ichan=band(1),band(2)
             tmodcounts = tmodcounts + speccoro_tot(ichan,i)* &
                  ratio_tot(ichan,i)*areaobs_tot(i)* &
                  expoobs_tot(i)
             tot1 = tot1 + speccoro_tot(ichan,i)
             tot2 = tot2 + ratio_tot(ichan,i)
          enddo
          write(errstr,*)i, tot1, tot2, areafwco_tot(i), &
               areaobs_tot(i), expoobs_tot(i)
          call message(AppMsg,Noisy)
          !
          ! Read in the fwc image old: im_name
          ! can evqpb replace this section???!!!
          if (withimg) then 
             ! old/new im_name/imgfwc_F
             ! mos1S002-im4-400-1250.fits check this!!!
             imgfwc_F = trim(adjustl(prefix)) // '-fwcimccd' //&
                  trim(adjustl(str_ccd)) // 'det-' // &
                  trim(adjustl(celow)) // '-' // &
                  trim(adjustl(cehigh)) // '.fits'
             write(errstr,*)"FWC image file name: ",trim(imgfwc_F)
             call message(AppMsg,Always)
             status = 0
             warndie = .FALSE. ! ok if not found or unreadable
             call check_open_set(imgfwc_F,imgfwc_S,1,status,warndie)
             !if (status == 1) then 
             !
             ! acts like retfit to get the array only
             !
             call read_2Darray(imgfwc_S,imgfwc_A,aDR32,aDR64,aDI16,imgfwc_JP,&
                  xSize,ySize,typeA)
             !
             ! fwc image used to be 780x780 now allocatable
             ! old/new image(780,780)/imgobs(xSize,ySize)
             if (i == 1) allocate(imgfwc(xSize,ySize),imgout(xSize,ySize))
             if ((xSize /= xGood) .or. (ySize /= yGood)) then
                write(errstr,*)"Read FWC image dims not = ",xGood,"x",yGood
                call error("badArrDims",trim(errstr))
             end if
             imgfwc(1:xSize,1:ySize) = imgfwc_JP(0:xSize-1,0:ySize-1)
             if(status == 0) then ! bhp: and do nuthin if not???!!!
                ! ss: Check for bright pixels
                tot = 0.0
                ntot = 0.0
                do j=1,xSize
                   do jj=1,ySize
                      if(imgfwc(j,jj) > 0) then
                         tot = tot + imgfwc(j,jj)
                         ntot = ntot + 1.0
                      endif
                   enddo
                enddo
                if(ntot > 0.0) then
                   avtot = tot/ntot
                   write(errstr,*) "Average non-zero value of FWC ", &
                        "image: ", avtot
                   call message(AppMsg,Always)
                   tot = 5.0*avtot
                   do j=1,xSize
                      do jj=1,ySize
                         if(imgfwc(j,jj) > tot) then
                            imgfwc(j,jj) = avtot
                         endif
                      enddo
                   enddo
                   tot = 0.0
                   do j=1,xSize
                      do jj=1,ySize
                         tot = tot + imgfwc(j,jj)
                      enddo
                   enddo
                   write(errstr,*) "FWC image scaling for chip: ", i, &
                        tmodcounts, tot, tmodcounts/tot
                   call message(AppMsg,Always)
                   do j=1,xSize
                      do jj=1,ySize
                         imgout(j,jj) = imgout(j,jj) + &
                              imgfwc(j,jj)*tmodcounts/tot
                      enddo
                   enddo
                endif
             endif
             call release(imgfwc_S)
          endif
       else
          write(errstr,*)"CCD#" // trim(str_ccd) // &
               " not included in background spectrum"
          call message(AppMsg,Always)
       endif
       if(withqdp) then ! bhp: boolean now was diag ge 2
          write(lunratio,*) 'no no no'
          write(lunaug,*)   'no no no'
          write(lunbacki,*) 'no no no'
          write(lunbacka,*) 'no no no'
       endif
    end do
    if(withqdp) then ! bhp: boolean now was diag ge 2
       close(lunratio) 
       close(lunaug) 
       close(lunbacki) 
       close(lunbacka)
    endif

    write(errstr,*)'Exposure for the FWC and observation data'
    call message(AppMsg,Sparse)
    do i=1,7
       if(ccds(i)) then ! bhp: boolean now
          write(errstr,4025) i,  expofwco_tot(i), expoobs_tot(i)
4025      format(1x,'Chip ',i1, ' exposure = ',f9.1, &
               ' exposure =',f9.1)
          call message(AppMsg,Sparse)              
       endif
    enddo
    !
    ! ss:  Search for and eliminate hot pixels
    ! old/new outim(780,780)/imgout(xSize,ySize)
    !
    if(withimg) then
       if(ipix == 1) then
          tot1 = 0.0
          tot2 = 0.0
          do i=1,xSize ! xSize/ySize can be anything not just 780x780
             do ii=1,ySize
                if(imgout(i,ii) > 0.) then
                   tot1 = tot1 + imgout(i,ii)
                   tot2 = tot2 + 1.0
                endif
             enddo
          enddo
          tot1 = 2.0*tot1/tot2
          do i=1,xSize
             do ii=1,ySize
                if(imgout(i,ii) > tot1) then
                   imgout(i,ii) = tot1
                endif
             enddo
          enddo
          tot1 = 0.0
          tot2 = 0.0
       endif
    endif
    !
    !  ss: Write out some more diagnostic information
    !
    if(withqdp) then ! bhp boolean now
       ! mos1S001-back.qdp
       qdpback_F = trim(adjustl(prefix)) // "-back.qdp"
       call open_ascii_file(trim(qdpback_F),lunqdpback)
       ! old/new 89(reused?)/lunqdpback
       write(lunqdpback,*) 'viewport 0.2 0.2 0.95 0.9'
       write(lunqdpback,*) 'read serr 2'
       write(lunqdpback,*) 'font roman'
       write(lunqdpback,*) 'cs 1.5'
       write(lunqdpback,*) 'log y'
       write(lunqdpback,*) 'la y Model Background Counts'
       write(lunqdpback,*) 'la x Channel'
    endif

    do ichan=1,chansO
       if(maskout(ichan) /= 0.0) then
          specout(ichan) = specout(ichan)/maskout(ichan)
          specout_U(ichan) = sqrt(specout_U(ichan))/maskout(ichan)
          if(withqdp) then ! bhp: boolean now
             write(lunqdpback,*) ichan, specout(ichan), specout_U(ichan)
          endif
       else
          specout(ichan) = 0.0
          specout_U(ichan) = 0.0
       endif
    enddo
    if(withqdp) then ! bhp: boolean now
       close(lunqdpback)
    endif
    !
    ! ss: Read in the observation image in detector coordinates
    !
    if (withimg) then       
       write(str_ccd,'(I1)')i
       ! old/new im_name/obimdet_F e.g. mos1S002-obj-im-det-400-1250.fits
       ! OLD: mos1S002-obj-im-det-400-1250.fits NEW: mos1S002-fovimtdet-400-1250.fits
       obimdet_F = trim(adjustl(prefix)) // '-fovimdet-' //&
            trim(adjustl(celow)) // '-' // &
            trim(adjustl(cehigh)) // '.fits'

       write(errstr,*)"Observation image in detector coords:"
       call message(AppMsg,Always)
       write(errstr,*)trim(obimdet_F)
       call message(AppMsg,Always)
       status = 0
       warndie = .TRUE. ! croak if not found OR found and SAS_CLOBBER set
       call check_open_set(obimdet_F,obimdet_S,1,status,warndie)
       !       allocate(obimdet(xSize,ySize))
       ! acts like retfit old/new image/obimdet
       ! returns xSize x ySize  pointer array
       call read_2Darray(obimdet_S,obimdet_A,aDR32,aDR64,aDI16,obimdet_JP,&
            xSize,ySize,typeA)
       if ((xSize /= xGood) .or. (ySize /= yGood)) then
          write(errstr,*)"Read OBJ Det image dims not = ",xGood,"x",yGood
          call error("badArrDims",trim(errstr))
       end if
       allocate(obimdet(xSize,ySize))
       obimdet(1:xSize,1:ySize) = obimdet_JP(0:xSize-1,0:ySize-1)
       tot1 = 0.0
       tot2 = 0.0
       do j=1,xSize
          do jj=1,ySize
             tot1 = tot1 + obimdet(j,jj) ! image
             tot2 = tot2 + imgout(j,jj)  ! outim
          enddo
       enddo
       write(errstr,*)'Total observation counts in band ',tot1
       call message(AppMsg,Always)
       write(errstr,*)'Total background counts in band ',tot2
       call message(AppMsg,Always)
       !
       !  Write out the background image
       !
       !  Create the output background image in det coords file
       !
       ! Create the output filename
       ! old/new imo_name/bkimdet_F, e.g. mos1S002-back-im-det-400-1250.fits
       ! OLD: mos1S002-back-im-det-400-1250.fits NEW mos1S002-bkimdet-400-1250.fits
       bkimdet_F = trim(adjustl(prefix)) // '-bkimdet-' //&
            trim(adjustl(celow)) // '-' // &
            trim(adjustl(cehigh)) // '.fits'
       write(errstr,*)"Background image in detector coords:"
       call message(AppMsg,Always)
       write(errstr,*)trim(bkimdet_F)
       call message(AppMsg,Always)
       !
       !  Create the output FITS file using the original file as a template
       !
       write(errstr,*)" Cloning obs image in det coords to output bkg image"
       call message(AppMsg,Always)
       call copyDataSet(trim(obimdet_F),trim(bkimdet_F))
       call release(obimdet_S)
       status = 0
       warndie = .TRUE. ! die if can't open and write to the file
       call check_open_set(bkimdet_F,bkimdet_S,4,status,warndie) ! 4='modify'
       !              
       !  Bridge the Al-Si gap
       !    Set a number of line-free windows
       !    Extract the data from those windows
       !    Fit a simple polynomial
       !    Use to interpolate over the Al-Si gap
       !
       !  The following two lines need to be converted to energies
       !
       nnn = 0 ! BHP: was 'n', making it too hard to grep for!!!
       allocate(tempe(chansO),temps(chansO),tempu(chansO))
       do i=1,nels
          do ichan=mine(i),maxe(i)
             if((ener(ichan) .gt. 0.1) .and. &
                  (specout(ichan) .gt. 0.0)) then
                nnn = nnn + 1
                tempe(nnn) = alog(ener(ichan))
                temps(nnn) = alog(specout(ichan))
                tempu(nnn) = 0.2
                write(30,*)tempe(nnn),temps(nnn),tempu(nnn)
             endif
          enddo
       enddo
       !
       ! BHP: poly_fit is in gofmatrix in task goflib
       ! how different is this from mssllib version 
       ! of the same name???!!!
       ! poly_fit returns coef
       !
       coef = 0.
       nord = 5
       call poly_fit(tempe,temps,tempu,nnn,nord,coef)
       
       if(withqdp) then
          bridge_F = trim(adjustl(prefix)) // '-bridge-fit.qdp'
          call open_ascii_file(trim(bridge_F),lunbridge)
          ! old/new 89/lunbridge
          write(lunbridge,*) 'viewport 0.2 0.2 0.95 0.9'
          write(lunbridge,*) 'read serr 2'
          write(lunbridge,*) 'font roman'
          write(lunbridge,*) 'cs 1.5'
          write(lunbridge,*) 'la y Arbitrary'
          write(lunbridge,*) 'la x Log(E) (keV)'
       endif
       allocate(modl(chansO))
       do ichan=1,chansO
          do i=0,nord
             modl(ichan) = modl(ichan) + &
                  coef(i+1)*(alog(ener(ichan))**i)
          enddo
          if(withqdp .and. (ener(ichan) > 0.1)) then
             write(lunbridge,*) alog(ener(ichan)), alog(specout(ichan)), &
                  0.05*(alog(specout(ichan))), modl(ichan)
          endif
       enddo
       if(withqdp) then
          close(lunbridge)
       endif
       !
       !  Finish the bridge
       !
       do ichan = plc_alsi(1),plc_alsi(2)
          fillin(ichan) = exp(modl(ichan))
          specout(ichan) = specout(ichan) + fillin(ichan)
       enddo
       do ichan = 7,chansO
          if ((specobs(ichan)-specobs_U(ichan) < ymin) .and. &
               (specobs(ichan)-specobs_U(ichan) > 1.0e-5)) &
               ymin = specobs(ichan)-specobs_U(ichan)
          if (specobs(ichan)+specobs_U(ichan) > ymax) &
               ymax = specobs(ichan)+specobs_U(ichan)
          if ((expoobs*backobs*(specout(ichan)-specout_U(ichan)) < ymin) &
               .and. (expoobs*backobs*(specout(ichan)-specout_U(ichan))  &
               > 1.0e-5)) &
               ymin = expoobs*backobs*(specout(ichan)-specout_U(ichan))
          if (expoobs*backobs*(specout(ichan)+specout_U(ichan)) > ymax) &
               ymax = expoobs*backobs*(specout(ichan)+specout_U(ichan))
       enddo
       ! write out to outqdp_F = mos1S002-spec.qdp, derived at top of program
       call open_ascii_file(trim(outQDP_F),lunqdp)
       write(lunoqdp,*) 'viewport 0.2 0.2 0.9 0.85'
       write(lunoqdp,*) 'la f'
       write(title,3110) obsidO, prefix
3110   format('"',i10,' -- MOS',a5,'"')
       write(lunoqdp,*) 'la t ', title
       write(lunoqdp,*) 'la x Energy (keV)'
       write(lunoqdp,*) 'la y Counts'
       i = 0.75*ymax
       write(lunoqdp,*) 'la 1 p 3 ', i, ' just left ', &
            '"Observed (red), Particle Background (green)"'
       write(lunoqdp,*) 'log y'
       write(lunoqdp,*) 'r x 0 12'
       write(lunoqdp,*) 'r y ', ymin, ymax
       write(lunoqdp,*) 'read serr 2'
       write(lunoqdp,*) 'read serr 3'
       write(lunoqdp,*) 'read serr'
       write(lunoqdp,*) 'font roman'
       write(lunoqdp,*) 'cs 1.8'
       do ichan = 1,chansO
          write(lunoqdp,*) ener(ichan), specobs(ichan), specobs_U(ichan), &
               expoobs*backobs*specout(ichan), &
               expoobs*backobs*specout_U(ichan)
       enddo
       close(lunoqdp)

    end if

    ! write out the spectra
    ! Create the output filename
    ! old/new imo_name/bkimdet_F, e.g. mos1S002-back-im-det-400-1250.fits
    backspecout_F = trim(adjustl(prefix)) // '-back.pi'
    write(errstr,*)"Creating output background spectra:",trim(backspecout_F)
    call message(AppMsg,Always)
    !
    !  Create the output FITS file using the original file as a template
    !
    call copyDataSet(obsSPEC_F,backspecout_F)
    status = 0
    warndie = .TRUE. ! croak if not found or unreadable
    call check_open_set(backspecout_F,backspec_S,4,status,warndie) ! 4=MODIFY
    !
    !
    !
    backspec_T =  table(backspec_S,"SPECTRUM",MODIFY)
    channel_C = column(backspec_T,"CHANNEL",MODIFY)
    countsR32_C = column(backspec_T,"COUNTS",MODIFY)
    !
    allocate(chansout(chansO))
    if(chisqfl) then ! write channel/counts/counts_err
       do ichan=1,chansO
          chansout(ichan) = ichan - 1
          specout(ichan) = specout(ichan) * expoobs * backobs
          specout_U(ichan) = specout_U(ichan) * expoobs * backobs
       enddo
    else ! replace R32 column with I32 column for counts
       allocate(ispecout(chansO))
       call deleteColumn(backspec_T,"COUNTS")
       countsI32_C = addColumn(backspec_T,"COUNTS",INTEGER32,units="channel",&
            comment= "physical units of field")       
       do ichan=1,chansO ! write channel/icounts only
          chansout(ichan) = ichan - 1
          ispecout(ichan) = nint(specout(ichan) * expoobs * backobs)
       enddo
    endif

    call release(backspec_S)
    
    return

  end subroutine mosback
  !
  !
  !
  subroutine printparams (inSPEC_F,outSPEC_F,inRMF_F,withqdp,outQDP_F,withimg,&
       outIMG_F,elow,ehigh,ccds,rev,obsid)
    !
    character(len=*) :: inSPEC_F, inRMF_F, outQDP_F, outIMG_F, outSPEC_F, obsid
    character(len=1) :: str_ccd
    integer :: elow, ehigh, rev, i
    logical :: withqdp, withimg, ccds(7)

    write(errstr,*) "Input parameters:"
    call message(AppMsg,Always)
    write(errstr,*) "Input Spectral file   : ",trim(inSPEC_F) ! old obj_file
    call message(AppMsg,Always)
    write(errstr,*) "Output Spectral file  : ",trim(outSPEC_F)! old back_file
    call message(AppMsg,Always)
    write(errstr,*) "Input RMF file        : ",trim(inRMF_F)  ! old obj_rmf
    call message(AppMsg,Always)
    write(errstr,*) "Create diagnostic IMAGE file (y/n)?: ",withimg
    call message(AppMsg,Always)
    if(withimg) then 
       write(errstr,*) "Output IMAGE file     : ",trim(outIMG_F) ! old imo_name
       call message(AppMsg,Always)
    endif
    write(errstr,*) "Create diagnostic QDP file (y/n)?: ",withqdp
    call message(AppMsg,Always)
    if (withqdp) then
       write(errstr,*) "Output QDP file       : ",trim(outQDP_F) ! old spec_file
       call message(AppMsg,Always)
    end if

    write(errstr,*) "Selected energy lower limit        : ",elow
    call message(AppMsg,Always)
    write(errstr,*) "Selected energy upper limit        : ",ehigh
    call message(AppMsg,Always)
    write(errstr,*) "Revolution                         : ",rev
    call message(AppMsg,Always)
    write(errstr,*) "OBSID                              : ",trim(adjustl(obsid))
    call message(AppMsg,Always)
    do i = 1, 7
       write(str_ccd,'(I1)')i
       write(errstr,*) "CCD(",str_ccd,") selected? (y/n)",ccds(i)
       call message(AppMsg,Always)
    end do
    
    return

  end subroutine printparams  

end module mosback_mod
